# Generated by Django 6.0 on 2026-01-06 01:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('schedule', '0014_use_autofields_for_pk'),
    ]

    operations = [
        migrations.RenameIndex(
            model_name='calendarrelation',
            new_name='schedule_ca_content_cddadb_idx',
            old_fields=('content_type', 'object_id'),
        ),
        migrations.RenameIndex(
            model_name='event',
            new_name='schedule_ev_start_a258cb_idx',
            old_fields=('start', 'end'),
        ),
        migrations.RenameIndex(
            model_name='eventrelation',
            new_name='schedule_ev_content_6ceecb_idx',
            old_fields=('content_type', 'object_id'),
        ),
        migrations.RenameIndex(
            model_name='occurrence',
            new_name='schedule_oc_start_76a2f8_idx',
            old_fields=('start', 'end'),
        ),
    ]
